@@ -1,5 +1,3 @@
-#!/pro/bin/perl
-
 package Config::Perl::V;
 
 use strict;
@@ -8,12 +6,12 @@ use warnings;
 use Config;
 use Exporter;
 use vars qw($VERSION @ISA @EXPORT_OK %EXPORT_TAGS);
-$VERSION     = "0.21";
-@ISA         = ("Exporter");
+$VERSION     = "0.32";
+@ISA         = qw( Exporter );
 @EXPORT_OK   = qw( plv2hash summary myconfig signature );
 %EXPORT_TAGS = (
-    all => [ @EXPORT_OK  ],
-    sig => [ "signature" ],
+    'all' => [ @EXPORT_OK  ],
+    'sig' => [ "signature" ],
     );
 
 #  Characteristics of this binary (from libperl):
@@ -26,13 +24,14 @@ $VERSION     = "0.21";
 #     perl -ne'(/^S_Internals_V/../^}/)&&s/^\s+"( .*)"/$1/ and print' perl.c
 #   perl.h line 4566 PL_bincompat_options
 #     perl -ne'(/^\w.*PL_bincompat/../^\w}/)&&s/^\s+"( .*)"/$1/ and print' perl.h
-my %BTD = map { $_ => 0 } qw(
+my %BTD = map {( $_ => 0 )} qw(
 
     DEBUGGING
     NO_HASH_SEED
     NO_MATHOMS
     NO_TAINT_SUPPORT
     PERL_BOOL_AS_CHAR
+    PERL_COPY_ON_WRITE
     PERL_DISABLE_PMC
     PERL_DONT_CREATE_GVSV
     PERL_EXTERNAL_GLOB
@@ -53,6 +52,7 @@ my %BTD = map { $_ => 0 } qw(
     PERL_MEM_LOG_STDERR
     PERL_MEM_LOG_TIMESTAMP
     PERL_NEW_COPY_ON_WRITE
+    PERL_OP_PARENT
     PERL_PERTURB_KEYS_DETERMINISTIC
     PERL_PERTURB_KEYS_DISABLED
     PERL_PERTURB_KEYS_RANDOM
@@ -60,14 +60,17 @@ my %BTD = map { $_ => 0 } qw(
     PERL_RELOCATABLE_INCPUSH
     PERL_USE_DEVEL
     PERL_USE_SAFE_PUTENV
+    SILENT_NO_TAINT_SUPPORT
     UNLINK_ALL_VERSIONS
     USE_ATTRIBUTES_FOR_PERLIO
     USE_FAST_STDIO
     USE_HASH_SEED_EXPLICIT
     USE_LOCALE
     USE_LOCALE_CTYPE
+    USE_NO_REGISTRY
     USE_PERL_ATOF
     USE_SITECUSTOMIZE
+    USE_THREAD_SAFE_LOCALE
 
     DEBUG_LEAKING_SCALARS
     DEBUG_LEAKING_SCALARS_FORK_DUMP
@@ -106,6 +109,7 @@ my %BTD = map { $_ => 0 } qw(
     USE_LOCALE_TIME
     USE_LONG_DOUBLE
     USE_PERLIO
+    USE_QUADMATH
     USE_REENTRANT_API
     USE_SFIO
     USE_SOCKS
@@ -156,7 +160,7 @@ my @config_vars = qw(
     useithreads usemultiplicity
     useperlio d_sfio uselargefiles usesocks
     use64bitint use64bitall uselongdouble
-    usemymalloc bincompat5005
+    usemymalloc default_inc_excludes_dot bincompat5005
 
     cc ccflags
     optimize
@@ -179,85 +183,103 @@ my @config_vars = qw(
     );
 
 my %empty_build = (
-    osname  => "",
-    stamp   => 0,
-    options => { %BTD },
-    patches => [],
+    'osname'  => "",
+    'stamp'   => 0,
+    'options' => { %BTD },
+    'patches' => [],
     );
 
-sub _make_derived
-{
+sub _make_derived {
     my $conf = shift;
 
-    for ( [ lseektype		=> "Off_t"	],
-	  [ myuname		=> "uname"	],
-	  [ perl_patchlevel	=> "patch"	],
+    for ( [ 'lseektype'		=> "Off_t"	],
+	  [ 'myuname'		=> "uname"	],
+	  [ 'perl_patchlevel'	=> "patch"	],
 	  ) {
-	my ($official, $derived) = @$_;
-	$conf->{config}{$derived}  ||= $conf->{config}{$official};
-	$conf->{config}{$official} ||= $conf->{config}{$derived};
-	$conf->{derived}{$derived} = delete $conf->{config}{$derived};
+	my ($official, $derived) = @{$_};
+	$conf->{'config'}{$derived}  ||= $conf->{'config'}{$official};
+	$conf->{'config'}{$official} ||= $conf->{'config'}{$derived};
+	$conf->{'derived'}{$derived} = delete $conf->{'config'}{$derived};
 	}
 
-    if (exists $conf->{config}{version_patchlevel_string} &&
-       !exists $conf->{config}{api_version}) {
-	my $vps = $conf->{config}{version_patchlevel_string};
+    if (exists $conf->{'config'}{'version_patchlevel_string'} &&
+       !exists $conf->{'config'}{'api_version'}) {
+	my $vps = $conf->{'config'}{'version_patchlevel_string'};
 	$vps =~ s{\b revision   \s+ (\S+) }{}x and
-	    $conf->{config}{revision}        ||= $1;
+	    $conf->{'config'}{'revision'}        ||= $1;
 
 	$vps =~ s{\b version    \s+ (\S+) }{}x and
-	    $conf->{config}{api_version}     ||= $1;
+	    $conf->{'config'}{'api_version'}     ||= $1;
 	$vps =~ s{\b subversion \s+ (\S+) }{}x and
-	    $conf->{config}{subversion}      ||= $1;
+	    $conf->{'config'}{'subversion'}      ||= $1;
 	$vps =~ s{\b patch      \s+ (\S+) }{}x and
-	    $conf->{config}{perl_patchlevel} ||= $1;
+	    $conf->{'config'}{'perl_patchlevel'} ||= $1;
 	}
 
-    ($conf->{config}{version_patchlevel_string} ||= join " ",
-	map  { ($_, $conf->{config}{$_} ) }
-	grep {      $conf->{config}{$_}   }
+    ($conf->{'config'}{'version_patchlevel_string'} ||= join " ",
+	map  { ($_, $conf->{'config'}{$_} ) }
+	grep {      $conf->{'config'}{$_}   }
 	qw( api_version subversion perl_patchlevel )) =~ s/\bperl_//; 
 
-    $conf->{config}{perl_patchlevel}  ||= "";	# 0 is not a valid patchlevel
+    $conf->{'config'}{'perl_patchlevel'}  ||= "";	# 0 is not a valid patchlevel
 
-    if ($conf->{config}{perl_patchlevel} =~ m{^git\w*-([^-]+)}i) {
-	$conf->{config}{git_branch}   ||= $1;
-	$conf->{config}{git_describe} ||= $conf->{config}{perl_patchlevel};
+    if ($conf->{'config'}{'perl_patchlevel'} =~ m{^git\w*-([^-]+)}i) {
+	$conf->{'config'}{'git_branch'}   ||= $1;
+	$conf->{'config'}{'git_describe'} ||= $conf->{'config'}{'perl_patchlevel'};
 	}
 
+    $conf->{'config'}{$_} ||= "undef" for grep m{^(?:use|def)} => @config_vars;
+
     $conf;
     } # _make_derived
 
-sub plv2hash
-{
+sub plv2hash {
     my %config;
 
     my $pv = join "\n" => @_;
 
-    if ($pv =~ m/^Summary of my\s+(\S+)\s+\(\s*(.*?)\s*\)/m) {
-	$config{"package"} = $1;
+    if ($pv =~ m{^Summary of my\s+(\S+)\s+\(\s*(.*?)\s*\)}m) {
+	$config{'package'} = $1;
 	my $rev = $2;
-	$rev =~ s/^ revision \s+ (\S+) \s*//x and $config{revision} = $1;
-	$rev and $config{version_patchlevel_string} = $rev;
-	my ($rel) = $config{"package"} =~ m{perl(\d)};
+	$rev =~ s/^ revision \s+ (\S+) \s*//x and $config{'revision'} = $1;
+	$rev and $config{'version_patchlevel_string'} = $rev;
+	my ($rel) = $config{'package'} =~ m{perl(\d)};
 	my ($vers, $subvers) = $rev =~ m{version\s+(\d+)\s+subversion\s+(\d+)};
 	defined $vers && defined $subvers && defined $rel and
-	    $config{version} = "$rel.$vers.$subvers";
+	    $config{'version'} = "$rel.$vers.$subvers";
 	}
 
-    if ($pv =~ m/^\s+(Snapshot of:)\s+(\S+)/) {
-	$config{git_commit_id_title} = $1;
-	$config{git_commit_id}       = $2;
+    if ($pv =~ m{^\s+(Snapshot of:)\s+(\S+)}) {
+	$config{'git_commit_id_title'} = $1;
+	$config{'git_commit_id'}       = $2;
 	}
 
-    if (my %kv = ($pv =~ m/\b(\w+)\s*=\s*('[^']+?'|\S+)/g)) {
+    # these are always last on line and can have multiple quotation styles
+    for my $k (qw( ccflags ldflags lddlflags )) {
+	$pv =~ s{, \s* $k \s*=\s* (.*) \s*$}{}mx or next;
+	my $v = $1;
+	$v =~ s/\s*,\s*$//;
+	$v =~ s/^(['"])(.*)\1$/$2/;
+	$config{$k} = $v;
+	}
+
+    if (my %kv = ($pv =~ m{\b
+	    (\w+)		# key
+	    \s*=		# assign
+	    ( '\s*[^']*?\s*'	# quoted value
+	    | \S+[^=]*?\s*\n	# unquoted running till end of line
+	    | \S+		# unquoted value
+	    | \s*\n		# empty
+	    )
+	    (?:,?\s+|\s*\n)?	# separator (5.8.x reports did not have a ','
+	    }gx)) {		# between every kv pair
 
 	while (my ($k, $v) = each %kv) {
-	    $k =~ s/\s+$//;
-	    $v =~ s/,$//;
-	    $v =~ m/^'(.*)'$/ and $v = $1;
-	    $v =~ s/^\s+//;
-	    $v =~ s/\s+$//;
+	    $k =~ s{\s+$}	{};
+	    $v =~ s{\s*\n\z}	{};
+	    $v =~ s{,$}		{};
+	    $v =~ m{^'(.*)'$} and $v = $1;
+	    $v =~ s{\s+$}	{};
 	    $config{$k} = $v;
 	    }
 	}
@@ -265,62 +287,68 @@ sub plv2hash
     my $build = { %empty_build };
 
     $pv =~ m{^\s+Compiled at\s+(.*)}m
-	and $build->{stamp}   = $1;
-    $pv =~ m{^\s+Locally applied patches:(?:\s+|\n)(.*)}m
-	and $build->{patches} = [ split m/\n+/, $1 ];
-    $pv =~ m{^\s+Compile-time options:(?:\s+|\n)(.*)}m
-	and map { $build->{options}{$_} = 1 } split m/\s+|\n/ => $1;
+	and $build->{'stamp'}   = $1;
+    $pv =~ m{^\s+Locally applied patches:(?:\s+|\n)(.*?)(?:[\s\n]+Buil[td] under)}ms
+	and $build->{'patches'} = [ split m{\n+\s*}, $1 ];
+    $pv =~ m{^\s+Compile-time options:(?:\s+|\n)(.*?)(?:[\s\n]+(?:Locally applied|Buil[td] under))}ms
+	and map { $build->{'options'}{$_} = 1 } split m{\s+|\n} => $1;
 
-    $build->{osname} = $config{osname};
+    $build->{'osname'} = $config{'osname'};
     $pv =~ m{^\s+Built under\s+(.*)}m
-	and $build->{osname}  = $1;
-    $config{osname} ||= $build->{osname};
+	and $build->{'osname'}  = $1;
+    $config{'osname'} ||= $build->{'osname'};
 
     return _make_derived ({
-	build		=> $build,
-	environment	=> {},
-	config		=> \%config,
-	derived		=> {},
-	inc		=> [],
+	'build'		=> $build,
+	'environment'	=> {},
+	'config'	=> \%config,
+	'derived'	=> {},
+	'inc'		=> [],
 	});
     } # plv2hash
 
-sub summary
-{
+sub summary {
     my $conf = shift || myconfig ();
-    ref $conf eq "HASH" &&
-	exists $conf->{config} && exists $conf->{build} or return;
+    ref $conf eq "HASH"
+    && exists $conf->{'config'}
+    && exists $conf->{'build'}
+    && ref $conf->{'config'} eq "HASH"
+    && ref $conf->{'build'}  eq "HASH" or return;
 
     my %info = map {
-	exists $conf->{config}{$_} ? ( $_ => $conf->{config}{$_} ) : () }
+	exists $conf->{'config'}{$_} ? ( $_ => $conf->{'config'}{$_} ) : () }
 	qw( archname osname osvers revision patchlevel subversion version
 	    cc ccversion gccversion config_args inc_version_list
 	    d_longdbl d_longlong use64bitall use64bitint useithreads
 	    uselongdouble usemultiplicity usemymalloc useperlio useshrplib 
 	    doublesize intsize ivsize nvsize longdblsize longlongsize lseeksize
+	    default_inc_excludes_dot
 	    );
-    $info{$_}++ for grep { $conf->{build}{options}{$_} } keys %{$conf->{build}{options}};
+    $info{$_}++ for grep { $conf->{'build'}{'options'}{$_} } keys %{$conf->{'build'}{'options'}};
 
     return \%info;
     } # summary
 
-sub signature
-{
-    eval { require Digest::MD5 };
-    $@ and return "00000000000000000000000000000000";
+sub signature {
+    my $no_md5 = "0" x 32;
+    my $conf = summary (shift) or return $no_md5;
 
-    my $conf = shift || summary ();
-    delete $conf->{config_args};
+    eval { require Digest::MD5 };
+    $@ and return $no_md5;
+
+    $conf->{'cc'} =~ s{.*\bccache\s+}{};
+    $conf->{'cc'} =~ s{.*[/\\]}{};
+
+    delete $conf->{'config_args'};
     return Digest::MD5::md5_hex (join "\xFF" => map {
 	"$_=".(defined $conf->{$_} ? $conf->{$_} : "\xFE");
-	} sort keys %$conf);
+	} sort keys %{$conf});
     } # signature
 
-sub myconfig
-{
+sub myconfig {
     my $args = shift;
-    my %args = ref $args eq "HASH"  ? %$args :
-               ref $args eq "ARRAY" ? @$args : ();
+    my %args = ref $args eq "HASH"  ? %{$args} :
+               ref $args eq "ARRAY" ? @{$args} : ();
 
     my $build = { %empty_build };
 
@@ -328,33 +356,34 @@ sub myconfig
     my $stamp = eval { Config::compile_date () };
     if (defined $stamp) {
 	$stamp =~ s/^Compiled at //;
-	$build->{osname}      = $^O;
-	$build->{stamp}       = $stamp;
-	$build->{patches}     =     [ Config::local_patches () ];
-	$build->{options}{$_} = 1 for Config::bincompat_options (),
-				      Config::non_bincompat_options ();
+	$build->{'osname'}      = $^O;
+	$build->{'stamp'}       = $stamp;
+	$build->{'patches'}     =     [ Config::local_patches () ];
+	$build->{'options'}{$_} = 1 for Config::bincompat_options (),
+					Config::non_bincompat_options ();
 	}
     else {
 	#y $pv = qx[$^X -e"sub Config::myconfig{};" -V];
 	my $cnf = plv2hash (qx[$^X -V]);
 
-	$build->{$_} = $cnf->{build}{$_} for qw( osname stamp patches options );
+	$build->{$_} = $cnf->{'build'}{$_} for qw( osname stamp patches options );
 	}
 
     my @KEYS = keys %ENV;
     my %env  =
-	map {      $_ => $ENV{$_} } grep m/^PERL/      => @KEYS;
-    $args{env} and
-	map { $env{$_} = $ENV{$_} } grep m{$args{env}} => @KEYS;
+	map {( $_ => $ENV{$_} )}  grep m{^PERL}        => @KEYS;
+    if ($args{'env'}) {
+	$env{$_}  =  $ENV{$_} for grep m{$args{'env'}} => @KEYS;
+	}
 
     my %config = map { $_ => $Config{$_} } @config_vars;
 
     return _make_derived ({
-	build		=> $build,
-	environment	=> \%env,
-	config		=> \%config,
-	derived		=> {},
-	inc		=> \@INC,
+	'build'		=> $build,
+	'environment'	=> \%env,
+	'config'	=> \%config,
+	'derived'	=> {},
+	'inc'		=> \@INC,
 	});
     } # myconfig
 
@@ -377,9 +406,9 @@ Config::Perl::V - Structured data retrieval of perl -V output
 
 =head2 $conf = myconfig ()
 
-This function will collect the data described in L<the hash structure> below,
+This function will collect the data described in L</"The hash structure"> below,
 and return that as a hash reference. It optionally accepts an option to
-include more entries from %ENV. See L<environment> below.
+include more entries from %ENV. See L</environment> below.
 
 Note that this will not work on uninstalled perls when called with
 C<-I/path/to/uninstalled/perl/lib>, but it works when that path is in
@@ -525,7 +554,7 @@ H.Merijn Brand <h.m.brand@xs4all.nl>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2009-2014 H.Merijn Brand
+Copyright (C) 2009-2020 H.Merijn Brand
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
